import { Atom, WritableAtom } from './atom';
import { SuspensePromise } from './suspensePromise';
declare type ResolveType<T> = T extends Promise<infer V> ? V : T;
declare type AnyAtomValue = unknown;
declare type AnyAtom = Atom<AnyAtomValue>;
declare type OnUnmount = () => void;
declare type ReadError = unknown;
declare type Revision = number;
declare type InvalidatedRevision = number;
declare type ReadDependencies = Map<AnyAtom, Revision>;
export declare type AtomState<Value = AnyAtomValue> = {
    r: Revision;
    i?: InvalidatedRevision;
    d: ReadDependencies;
} & ({
    e: ReadError;
} | {
    p: SuspensePromise;
} | {
    v: ResolveType<Value>;
});
export declare type VersionObject = {
    p?: VersionObject;
};
declare type Listeners = Set<(version?: VersionObject) => void>;
declare type Dependents = Set<AnyAtom>;
declare type Mounted = {
    l: Listeners;
    t: Dependents;
    u?: OnUnmount;
};
declare type StateListener = () => void;
export declare const READ_ATOM = "r";
export declare const WRITE_ATOM = "w";
export declare const COMMIT_ATOM = "c";
export declare const SUBSCRIBE_ATOM = "s";
export declare const RESTORE_ATOMS = "h";
export declare const DEV_SUBSCRIBE_STATE = "n";
export declare const DEV_GET_MOUNTED_ATOMS = "l";
export declare const DEV_GET_ATOM_STATE = "a";
export declare const DEV_GET_MOUNTED = "m";
export declare const createStore: (initialValues?: Iterable<readonly [
    AnyAtom,
    unknown
]> | undefined) => {
    r: <Value>(readingAtom: Atom<Value>, version?: VersionObject | undefined) => AtomState<Value>;
    w: <Value_1, Update, Result extends void | Promise<void>>(writingAtom: WritableAtom<Value_1, Update, Result>, update: Update, version?: VersionObject | undefined) => void | Promise<void>;
    c: (_atom: AnyAtom | null, version?: VersionObject | undefined) => void;
    s: (atom: AnyAtom, callback: (version?: VersionObject | undefined) => void) => () => void;
    h: (values: Iterable<readonly [
        AnyAtom,
        AnyAtomValue
    ]>, version?: VersionObject | undefined) => void;
    n: (l: StateListener) => () => void;
    l: () => IterableIterator<AnyAtom>;
    a: (a: AnyAtom) => AtomState<unknown> | undefined;
    m: (a: AnyAtom) => Mounted | undefined;
} | {
    r: <Value>(readingAtom: Atom<Value>, version?: VersionObject | undefined) => AtomState<Value>;
    w: <Value_1, Update, Result extends void | Promise<void>>(writingAtom: WritableAtom<Value_1, Update, Result>, update: Update, version?: VersionObject | undefined) => void | Promise<void>;
    c: (_atom: AnyAtom | null, version?: VersionObject | undefined) => void;
    s: (atom: AnyAtom, callback: (version?: VersionObject | undefined) => void) => () => void;
    h: (values: Iterable<readonly [
        AnyAtom,
        AnyAtomValue
    ]>, version?: VersionObject | undefined) => void;
    n?: never;
    l?: never;
    a?: never;
    m?: never;
};
export declare type Store = ReturnType<typeof createStore>;
export {};
